@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: common.c,v 1.30 2005/08/17 02:24:29 evertonm Exp $ */
+/* $Id: common.c,v 1.36 2005/09/10 22:33:17 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -40,7 +40,7 @@ const char * const NEPIM_LABEL_TOTAL_BROKEN = "avg(broken)";
 
 const char *nepim_version()
 {
-  return "0.11";
+  return "0.12";
 }
 
 static void nepim_dump_stat(FILE *out, const char *label,
@@ -59,7 +59,7 @@ static void nepim_dump_stat(FILE *out, const char *label,
   assert(!result);
 
   fprintf(out,
-	  ": %s %ld/%ld kbps_in=%.2f kbps_out=%.2f rcv/s=%.2f snd/s=%.2f\n", 
+	  ": %s %ld/%ld kbps_in=%.2f kbps_out=%.2f rcv/s=%.2f snd/s=%.2f", 
 	  label,
 	  now.tv_sec - sec_start,
 	  sec_duration,
@@ -79,19 +79,28 @@ void nepim_pipe_stat(FILE *out, const char *label, int sd,
   nepim_dump_stat(out, label, bytes_recv, bytes_sent, 
 		  interval, sec_start, sec_duration,
 		  reads, writes);
+
+  fprintf(out, "\n");
 }
 
 void nepim_slot_stat(FILE *out, const char *label, int sd,
 		     int local_slot, int remote_slot,
 		     long long bytes_recv, long long bytes_sent, 
 		     float interval, long sec_start, 
-		     long sec_duration, int reads, int writes)
+		     long sec_duration, int reads, int writes,
+		     int pkt_lost, int pkt_dup)
 {
   fprintf(out, "%d %d-%d", sd, local_slot, remote_slot);
 
   nepim_dump_stat(out, label, bytes_recv, bytes_sent, 
 		  interval, sec_start, sec_duration,
 		  reads, writes);
+
+#if 0
+  fprintf(out, " lost=%d dup=%d", pkt_lost, pkt_dup);
+#endif
+
+  fprintf(out, "\n");
 }
 
 void report_broken_pipe_stat(FILE *out, const nepim_pipe_t *pipe)
@@ -151,13 +160,15 @@ void report_broken_slot_stat(FILE *out, const nepim_slot_t *slot)
 		  session->tv_start.tv_sec,
 		  session->test_duration,
 		  session->total_reads,
-		  session->total_writes);
+		  session->total_writes,
+		  slot->total_pkt_lost,
+		  slot->total_pkt_dup);
 }
 
 #define NEPIM_MEGA (1000000)
 #define NEPIM_8MEGA (8000000)
 
-void nepim_timer_usec_add(struct timeval *tv, suseconds_t usec)
+void nepim_timer_usec_add(struct timeval *tv, susec_t usec)
 {
   tv->tv_usec += usec;
 
@@ -171,7 +182,7 @@ void nepim_timer_usec_add(struct timeval *tv, suseconds_t usec)
   assert(tv->tv_usec < NEPIM_MEGA);
 }
 
-long long nepim_bps2bytes(long long bps_bit_rate, suseconds_t usec_delay)
+long long nepim_bps2bytes(long long bps_bit_rate, susec_t usec_delay)
 {
   long long bytes;
 
@@ -182,7 +193,7 @@ long long nepim_bps2bytes(long long bps_bit_rate, suseconds_t usec_delay)
   return bytes;
 }
 
-long long nepim_min_bps(suseconds_t usec_delay)
+long long nepim_min_bps(susec_t usec_delay)
 {
   long long rate;
 
@@ -192,18 +203,21 @@ long long nepim_min_bps(suseconds_t usec_delay)
   return rate;
 }
 
-int nepim_pps2packets(int pps_pkt_rate, suseconds_t usec_delay)
+int nepim_pps2packets(int pps_pkt_rate, susec_t usec_delay)
 {
-  int pkts;
+  long long pkts;
 
   pkts = pps_pkt_rate;
   pkts *= usec_delay;
   pkts /= NEPIM_MEGA;
 
+  assert(pkts > -2000000000);
+  assert(pkts < 2000000000);
+
   return pkts;
 }
 
-int nepim_min_pps(suseconds_t usec_delay)
+int nepim_min_pps(susec_t usec_delay)
 {
   int pps;
 
