@@ -24,6 +24,8 @@ Boston, MA 02111-1307, USA.
 
 #include <oop.h>
 
+#include "usec.h"
+
 typedef struct nepim_global_t nepim_global_t;
 
 struct nepim_global_t {
@@ -44,19 +46,22 @@ struct nepim_global_t {
   int         udp_read_size;
   const char *portname;
   int         no_inet6;
-  suseconds_t write_delay;
+  susec_t     write_delay;
   int         listen_backlog;
   int         pmtu_mode;
   char       *bind_list;
   char       *join_list;
   int         ttl;
   int         mcast_ttl;
-  suseconds_t udp_greet_delay; /* usec */
+  susec_t     udp_greet_delay; /* usec */
   int         max_greetings;
-  suseconds_t udp_keepalive_timer; /* usec, receivers */
-  suseconds_t udp_keepalive_delay; /* usec, non-senders */
+  susec_t     udp_keepalive_timer; /* usec, receivers */
+  susec_t     udp_keepalive_delay; /* usec, non-senders */
   int         udp_keepalive_require; /* boolean, receivers */
   int         udp_keepalive_send; /* boolean, non-senders */
+  int         udp_win_max;
+  int         win_recv;
+  int         win_send;
 
   oop_source_sys *oop_sys;
   oop_source     *oop_src;
