@@ -21,7 +21,7 @@ Boston, MA 02111-1307, USA.
 /*
   nepim - network pipemeter
 
-  $Id: main.c,v 1.18 2005/08/17 02:24:29 evertonm Exp $
+  $Id: main.c,v 1.22 2005/09/09 22:40:37 evertonm Exp $
  */
 
 #include <stdlib.h>
@@ -31,13 +31,13 @@ Boston, MA 02111-1307, USA.
 #include <oop.h>
 #include <assert.h>
 
-#ifdef NEPIM_SOLARIS
+#ifdef HAVE_SIGHANDLER_T
+# define __USE_GNU
+# include <signal.h>
+#else
 # include <signal.h>
 typedef void (*sighandler_t)(int);
-#else
-# define __USE_GNU /* for sighandler_t in <signal.h> */
-# include <signal.h>
-#endif /* NEPIM_SOLARIS */
+#endif
 
 #include "conf.h"
 #include "common.h"
@@ -91,9 +91,11 @@ static void usage(FILE *out, const char *prog_name)
 	  "  -z size       TCP read size (defaults to %d bytes)\n"
 	  "  -W size       UDP write size (defaults to %d bytes)\n"
 	  "  -Z size       UDP read size (defaults to %d bytes)\n"
+	  "  -O size       maximum socket send buffer in bytes\n"
+	  "  -I size       maximum socket receive buffer in bytes\n"
 	  "  -6            disable IPv6\n"
 	  "client-only options:\n"
-	  "  -u            UDP mode (defaults to tcp)\n"
+	  "  -u            UDP mode (defaults to TCP)\n"
 	  "  -s            client-send simplex mode (defaults to server-send)\n"
 	  "  -d            duplex traffic (defaults to simplex)\n"
 	  "  -c host-list  client mode (defaults to server mode)\n"
@@ -284,15 +286,15 @@ int main(int argc, const char *argv[])
 		nepim_global.prog_name);
 	exit(1);
       }
-      nepim_global.bit_rate = atoll(argv[i]);
+      nepim_global.bit_rate = strtoll(argv[i], 0, 10);
       bytes = nepim_bps2bytes(nepim_global.bit_rate,
 			      nepim_global.write_delay);
       if (bytes < 1) {
 	long long min_bps = nepim_min_bps(nepim_global.write_delay);
 	fprintf(stderr,
-		"%s: bit-rate=%lld lower than minimum=%lld\n",
-		nepim_global.prog_name, nepim_global.bit_rate,
-		min_bps);
+		"%s: bit-rate=%lld (bytes=%d) lower than minimum=%lld\n",
+		nepim_global.prog_name,
+		nepim_global.bit_rate, bytes, min_bps);
 	exit(1);
       }
       assert(nepim_global.bit_rate >= nepim_min_bps(nepim_global.write_delay));
@@ -313,9 +315,9 @@ int main(int argc, const char *argv[])
       if (pkts < 1) {
 	int min_pps = nepim_min_pps(nepim_global.write_delay);
 	fprintf(stderr,
-		"%s: pkt-rate=%d lower than minimum=%d\n",
-		nepim_global.prog_name, nepim_global.pkt_rate,
-		min_pps);
+		"%s: pkt-rate=%d (pkts=%d) lower than minimum=%d\n",
+		nepim_global.prog_name, 
+		nepim_global.pkt_rate, pkts, min_pps);
 	exit(1);
       }
       assert(nepim_global.pkt_rate >= nepim_min_pps(nepim_global.write_delay));
@@ -421,6 +423,28 @@ int main(int argc, const char *argv[])
       continue;
     }
 
+    if (!strcmp(arg, "-O")) {
+      ++i;
+      if (i >= argc) {
+	fprintf(stderr, "%s: missing maximum socket send buffer size\n",
+		nepim_global.prog_name);
+	exit(1);
+      }
+      nepim_global.win_send = atoi(argv[i]);
+      continue;
+    }
+
+    if (!strcmp(arg, "-I")) {
+      ++i;
+      if (i >= argc) {
+	fprintf(stderr, "%s: missing maximum socket read buffer size\n",
+		nepim_global.prog_name);
+	exit(1);
+      }
+      nepim_global.win_recv = atoi(argv[i]);
+      continue;
+    }
+
     fprintf(stderr, "%s: unknown option: %s\n",
 	    nepim_global.prog_name, arg);
     usage(stderr, nepim_global.prog_name);
