@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: pipe.h,v 1.5 2005/07/26 11:41:50 evertonm Exp $ */
+/* $Id: pipe.h,v 1.6 2005/08/15 17:41:17 evertonm Exp $ */
 
 #ifndef NEPIM_PIPE_H
 #define NEPIM_PIPE_H
@@ -44,6 +44,8 @@ void nepim_pipe_set_init(nepim_pipe_set_t *set);
 nepim_pipe_t *nepim_pipe_set_get(const nepim_pipe_set_t *set, int sd);
 
 void nepim_pipe_set_add(nepim_pipe_set_t *set, int sd, 
+			const struct sockaddr *remote,
+			socklen_t remote_len,
 			const nepim_greet_t *opt);
 
 void nepim_pipe_set_del(nepim_pipe_set_t *set, int sd);
