@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: server.c,v 1.36 2005/08/17 02:24:29 evertonm Exp $ */
+/* $Id: server.c,v 1.38 2005/09/12 23:41:41 evertonm Exp $ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -607,6 +607,8 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
   int result;
   int tcp_listeners = 0;
 
+  memset(&hints, 0, sizeof(hints));
+
   hints.ai_socktype = SOCK_STREAM;
   hints.ai_protocol = IPPROTO_TCP;
   hints.ai_flags = AI_CANONNAME;
@@ -642,7 +644,9 @@ static int spawn_tcp_listener(const char *hostname, const char *portname)
 				      ai->ai_protocol,
 				      nepim_global.listen_backlog,
 				      nepim_global.pmtu_mode,
-				      nepim_global.ttl);
+				      nepim_global.ttl,
+				      nepim_global.win_recv,
+				      nepim_global.win_send);
     if (sd < 0) {
       fprintf(stderr, 
 	      "%s: TCP listener socket failed for %s,%s(%d): %d\n",
