@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: slot.c,v 1.23 2005/08/16 12:41:46 evertonm Exp $ */
+/* $Id: slot.c,v 1.24 2005/08/18 21:59:11 evertonm Exp $ */
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -73,10 +73,17 @@ void nepim_slot_set_add(nepim_slot_set_t *set, int sd,
   slot->greetings_sent        = 0;
   slot->client_writer_status  = NEPIM_SLOT_CLIENT_GREET;
   slot->keepalives_recv       = 0;
+
+  slot->interval_pkt_lost = 0;
+  slot->interval_pkt_dup  = 0;
+  slot->total_pkt_lost    = 0;
+  slot->total_pkt_dup     = 0;
   
   nepim_session_init(&slot->session, opt, 
 		     remote, remote_len,
 		     SESSION_SLOT, index);
+
+  nepim_win_init(&slot->win_recv_seq, nepim_global.udp_win_max);
   
   nepim_array_add(&set->array, index, slot);
 }
@@ -85,6 +92,7 @@ void nepim_slot_set_del(nepim_slot_set_t *set, int index)
 {
   nepim_slot_t *slot = nepim_slot_set_get(set, index);
   assert(slot);
+  nepim_win_del(&slot->win_recv_seq);
   free(slot);
   nepim_array_del(&set->array, index);
 }
