@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: slot.h,v 1.20 2005/08/15 17:41:17 evertonm Exp $ */
+/* $Id: slot.h,v 1.21 2005/08/18 21:59:11 evertonm Exp $ */
 
 #ifndef NEPIM_SLOT_H
 #define NEPIM_SLOT_H
@@ -28,6 +28,7 @@ Boston, MA 02111-1307, USA.
 
 #include "array.h"
 #include "session.h"
+#include "win.h"
 
 typedef struct nepim_slot_t nepim_slot_t;
 typedef struct nepim_slot_set_t nepim_slot_set_t;
@@ -47,6 +48,11 @@ struct nepim_slot_t {
   int             greetings_sent;       /* counter */
   int             client_writer_status; /* greet/send */
   int             keepalives_recv;      /* counter */
+  nepim_win_t     win_recv_seq;
+  int             interval_pkt_lost;
+  int             interval_pkt_dup;
+  int             total_pkt_lost;
+  int             total_pkt_dup;
 };
 
 struct nepim_slot_set_t {
