@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: sock.c,v 1.18 2005/08/05 11:12:13 evertonm Exp $ */
+/* $Id: sock.c,v 1.23 2005/09/12 23:21:48 evertonm Exp $ */
 
 
 #include <assert.h>
@@ -37,9 +37,6 @@ Boston, MA 02111-1307, USA.
 
 #include "sock.h"
 
-/* 
-   Solaris
-*/
 #ifndef SOL_IP
 #define SOL_IP 0
 #endif
@@ -49,13 +46,30 @@ Boston, MA 02111-1307, USA.
 #ifndef SOL_TCP
 #define SOL_TCP 6
 #endif
-
-/* 
-   Supposedly from <netinet/ip.h> 
-*/
 #ifndef IP_MTU
 #define IP_MTU 14
 #endif
+#ifndef IP_ADD_MEMBERSHIP
+#define IP_ADD_MEMBERSHIP 35
+#endif
+#ifndef IPV6_ADD_MEMBERSHIP
+#define IPV6_ADD_MEMBERSHIP 20
+#endif
+
+#ifndef HAVE_IP_MREQN
+struct ip_mreqn {
+  struct in_addr imr_multiaddr;
+  struct in_addr imr_address;
+  int            imr_ifindex;
+};
+#endif
+
+#ifndef HAVE_IPV6_MREQ
+struct ipv6_mreq {
+  struct in6_addr ipv6mr_multiaddr;
+  int             ipv6mr_ifindex;
+};
+#endif
 
 #define NEPIM_SOCK_ERR_NONE           (0)
 #define NEPIM_SOCK_ERR_UNSPEC         (-1)
@@ -72,6 +86,8 @@ Boston, MA 02111-1307, USA.
 #define NEPIM_SOCK_ERR_TTL            (-12)
 #define NEPIM_SOCK_ERR_MCAST_TTL      (-13)
 #define NEPIM_SOCK_ERR_MCAST_JOIN     (-14)
+#define NEPIM_SOCK_ERR_WIN_RECV       (-15)
+#define NEPIM_SOCK_ERR_WIN_SEND       (-16)
 
 int nepim_sock_get_port(const struct sockaddr *addr)
 {
@@ -86,6 +102,61 @@ int nepim_sock_get_port(const struct sockaddr *addr)
   return ntohs(sa->inet.sin_port);
 }
 
+int nepim_sock_family(const struct sockaddr *addr)
+{
+  union {
+    struct sockaddr_in inet;
+    struct sockaddr_in6 inet6;
+  } *sa = (void *) addr;
+
+  assert(&(sa->inet.sin_family) == &(sa->inet6.sin6_family));
+  assert(sa->inet.sin_family == sa->inet6.sin6_family);
+
+  return sa->inet.sin_family;
+}
+
+void nepim_sock_dump_addr(char *buf, int buf_size, const struct sockaddr *addr)
+{
+  union {
+    struct sockaddr_in inet;
+    struct sockaddr_in6 inet6;
+  } *sa = (void *) addr;
+  const char *dst;
+  int family;
+
+  family = nepim_sock_family(addr);
+
+  assert(PF_INET == AF_INET);
+  assert(PF_INET6 == AF_INET6);
+
+  switch (family) {
+  case PF_INET:
+    assert(buf_size >= INET_ADDRSTRLEN);
+    dst = inet_ntop(family, &sa->inet.sin_addr, buf, buf_size);
+    break;
+  case PF_INET6:
+    assert(buf_size >= INET6_ADDRSTRLEN);
+    dst = inet_ntop(family, &sa->inet6.sin6_addr, buf, buf_size);
+    break;
+  default:
+    assert(0);
+  }
+
+#ifndef NDEBUG
+  {
+    if (!dst) {
+      fprintf(stderr,
+	      "%s %s: inet_ntop() failure: errno=%d %s\n",
+	      __FILE__, __PRETTY_FUNCTION__,
+	      errno, strerror(errno));
+    }
+  }
+#endif /* NDEBUG */
+
+  assert(dst);
+  assert(dst == buf);
+}
+
 int nepim_socket_block(int sd)
 {
   long flags;
@@ -134,15 +205,49 @@ int nepim_socket_ttl(int sd, int ttl)
   return setsockopt(sd, SOL_IP, IP_TTL, &ttl, sizeof(ttl));
 }
 
-int nepim_socket_mcast_ttl(int sd, int mcast_ttl)
+int nepim_socket_mcast_ttl(int sd, int mc_ttl)
 {
-  if (mcast_ttl < 0)
+#ifdef HAVE_UCHAR_MCAST_TTL
+  unsigned char mcast_ttl;
+#else
+  int mcast_ttl;
+#endif
+
+  if (mc_ttl < 0)
     return NEPIM_SOCK_ERR_NONE;
 
+  assert(mc_ttl >= 0);
+  assert(mc_ttl < 256);
+
+  mcast_ttl = mc_ttl;
+
   return setsockopt(sd, SOL_IP, IP_MULTICAST_TTL, &mcast_ttl, sizeof(mcast_ttl));
 }
 
-static int create_socket(int domain, int type, int protocol, int pmtu_mode, int ttl)
+static int socket_set_win_recv(int sd, int win_recv)
+{
+  if (win_recv < 0)
+    return NEPIM_SOCK_ERR_NONE;
+
+  if (setsockopt(sd, SOL_SOCKET, SO_RCVBUF, &win_recv, sizeof(win_recv)))
+    return NEPIM_SOCK_ERR_WIN_RECV;
+
+  return NEPIM_SOCK_ERR_NONE;
+}
+
+static int socket_set_win_send(int sd, int win_send)
+{
+  if (win_send < 0)
+    return NEPIM_SOCK_ERR_NONE;
+
+  if (setsockopt(sd, SOL_SOCKET, SO_SNDBUF, &win_send, sizeof(win_send)))
+    return NEPIM_SOCK_ERR_WIN_SEND;
+  
+  return NEPIM_SOCK_ERR_NONE;
+}
+
+static int create_socket(int domain, int type, int protocol, int pmtu_mode, int ttl,
+			 int win_recv, int win_send)
 {
   int sd;
   int result;
@@ -165,12 +270,23 @@ static int create_socket(int domain, int type, int protocol, int pmtu_mode, int
     return result;
   }
 
+  result = socket_set_win_recv(sd, win_recv);
+  if (result) {
+    close(sd);
+    return result;
+  }
+
+  result = socket_set_win_send(sd, win_send);
+  if (result) {
+    close(sd);
+    return result;
+  }
+
   return sd;
 }
 
 static int socket_mcast_join(int sd, int family, struct sockaddr *addr, int addr_len)
 {
-#ifndef NEPIM_SOLARIS
   union {
     struct sockaddr_in inet;
     struct sockaddr_in6 inet6;
@@ -207,7 +323,6 @@ static int socket_mcast_join(int sd, int family, struct sockaddr *addr, int addr
   }
 
   assert(0);
-#endif /* NEPIM_SOLARIS */
 
   return NEPIM_SOCK_ERR_MCAST_JOIN;
 }
@@ -219,12 +334,15 @@ int nepim_create_socket(struct sockaddr *addr,
 			int protocol,
 			int pmtu_mode,
 			int ttl,
-			int mcast_join)
+			int mcast_join,
+			int win_recv,
+			int win_send)
 {
   int sd;
   int result;
 
-  sd = create_socket(family, type, protocol, pmtu_mode, ttl);
+  sd = create_socket(family, type, protocol, pmtu_mode, ttl,
+		     win_recv, win_send);
   if (sd < 0)
     return sd;
 
@@ -244,27 +362,11 @@ int nepim_create_socket(struct sockaddr *addr,
     return result;
   }
 
-#if 0
-  fprintf(stderr, "DEBUG %s %s bind: sd=%d af=%d proto=%d addr=%s port=%d\n",
-	  __FILE__, __PRETTY_FUNCTION__, sd, 
-	  family, protocol,
-	  inet_ntoa(((struct sockaddr_in *) addr)->sin_addr),
-	  ntohs(((struct sockaddr_in *) addr)->sin_port));
-#endif
-
   if (bind(sd, addr, addr_len)) {
     close(sd);
     return NEPIM_SOCK_ERR_BIND;
   }
 
-#if 0
-  fprintf(stderr, "DEBUG %s %s bound: sd=%d af=%d proto=%d addr=%s port=%d\n",
-	  __FILE__, __PRETTY_FUNCTION__, sd, 
-	  family, protocol,
-	  inet_ntoa(((struct sockaddr_in *) addr)->sin_addr),
-	  ntohs(((struct sockaddr_in *) addr)->sin_port));
-#endif
-
   return sd;
 }
 
@@ -275,13 +377,15 @@ int nepim_create_listener_socket(struct sockaddr *addr,
 				 int protocol,
 				 int backlog,
 				 int pmtu_mode,
-				 int ttl)
+				 int ttl,
+				 int win_recv,
+				 int win_send)
 {
   int sd;
 
   sd = nepim_create_socket(addr, addr_len, family,
 			   type, protocol, pmtu_mode, 
-			   ttl, 0);
+			   ttl, 0, win_recv, win_send);
   if (sd < 0)
     return sd;
 
@@ -348,12 +452,15 @@ int nepim_connect_client_socket(struct sockaddr *addr,
 				int type,
 				int protocol,
 				int pmtu_mode,
-				int ttl)
+				int ttl,
+				int win_recv,
+				int win_send)
 {
   int sd;
   int result;
 
-  sd = create_socket(family, type, protocol, pmtu_mode, ttl);
+  sd = create_socket(family, type, protocol, pmtu_mode, ttl,
+		     win_recv, win_send);
   if (sd < 0)
     return sd;
 
@@ -437,7 +544,7 @@ int nepim_socket_get_ttl(int sd)
 
 static int socket_mcast_get_ttl(int sd)
 {
-#ifdef NEPIM_SOLARIS
+#ifdef HAVE_UCHAR_MCAST_TTL
   unsigned char mcast_ttl;
 #else
   int mcast_ttl;
@@ -452,19 +559,49 @@ static int socket_mcast_get_ttl(int sd)
   return mcast_ttl;
 }
 
+static int socket_get_win_recv(int sd)
+{
+  int win_recv;
+  socklen_t optlen = sizeof(win_recv);
+
+  if (getsockopt(sd, SOL_SOCKET, SO_RCVBUF, &win_recv, &optlen))
+    return NEPIM_SOCK_ERR_WIN_RECV;
+  
+  assert(optlen == sizeof(win_recv));
+
+  return win_recv;
+}
+
+static int socket_get_win_send(int sd)
+{
+  int win_send;
+  socklen_t optlen = sizeof(win_send);
+
+  if (getsockopt(sd, SOL_SOCKET, SO_SNDBUF, &win_send, &optlen))
+    return NEPIM_SOCK_ERR_WIN_SEND;
+  
+  assert(optlen == sizeof(win_send));
+
+  return win_send;
+}
+
 void nepim_sock_show_opt(FILE *out, int sd)
 {
   int pmtud_mode;
   int mtu;
   int ttl;
   int mcast_ttl;
+  int win_recv;
+  int win_send;
 
   pmtud_mode = nepim_socket_pmtu_get_mode(sd);
   mtu = nepim_socket_pmtu_get_mtu(sd);
   ttl = nepim_socket_get_ttl(sd);
   mcast_ttl = socket_mcast_get_ttl(sd);
+  win_recv = socket_get_win_recv(sd);
+  win_send = socket_get_win_send(sd);
 
   fprintf(out, 
-	  "%d: pmtud_mode=%d path_mtu=%d ttl=%d mcast_ttl=%d\n",
-	  sd, pmtud_mode, mtu, ttl, mcast_ttl);
+	  "%d: pmtud_mode=%d path_mtu=%d ttl=%d mcast_ttl=%d win_recv=%d win_send=%d\n",
+	  sd, pmtud_mode, mtu, ttl, mcast_ttl, win_recv, win_send);
 }
