@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: udp_client.c,v 1.36 2005/08/17 02:24:29 evertonm Exp $ */
+/* $Id: udp_client.c,v 1.40 2005/09/12 23:41:41 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -102,26 +102,6 @@ static void *on_udp_keepalive_require(oop_source *src, struct timeval tv, void *
   return OOP_CONTINUE;
 }
 
-#if 0
-static void schedule_keepalive_timer(nepim_slot_t *slot)
-{
-  nepim_session_t *session = &slot->session;
-
-  assert(session->udp_keepalive_require);
-
-  {
-    int result = gettimeofday(&session->tv_keepalive_timer, 0);
-    assert(!result);
-  }
-  nepim_timer_usec_add(&session->tv_keepalive_timer, 
-		       nepim_global.udp_keepalive_timer);
-
-  nepim_global.oop_src->on_time(nepim_global.oop_src,
-				session->tv_keepalive_timer,
-				on_udp_keepalive_require, slot);
-}
-#endif
-
 static void *on_udp_keepalive_time(oop_source *src, struct timeval tv, void *user)
 {
   nepim_slot_t *slot = user;
@@ -194,22 +174,17 @@ static void udp_slot_start(nepim_slot_t *slot)
     nepim_schedule_keepalive_timer(slot, on_udp_keepalive_require);
 }
 
-static void udp_consume(nepim_slot_t *slot, size_t len)
+static void udp_consume(nepim_slot_t *slot, size_t len, uint32_t seq)
 {
   nepim_session_t *session = &slot->session;
 
-#if 0
-  fprintf(stderr, 
-	  "DEBUG FIXME %s %s: %d %d-%d: in_seq=%d len=%d\n",
-	  __FILE__, __PRETTY_FUNCTION__,
-	  slot->udp_sd, slot->index, slot->index_remote,
-	  slot->seq, len);
-#endif
-
   /* any valid traffic accounted as keepalive */
   ++slot->keepalives_recv;
 
   nepim_session_read_add(session, len);
+
+  nepim_win_add(&slot->win_recv_seq, seq,
+		&slot->interval_pkt_lost, &slot->interval_pkt_dup);
 }
 
 static void *on_udp_read_segment(oop_source *src, int sd,
@@ -321,7 +296,7 @@ static void *on_udp_read_segment(oop_source *src, int sd,
       return OOP_CONTINUE;
     }
 
-    udp_consume(slot, rd);
+    udp_consume(slot, rd, hdr.seq);
   }
 
   return OOP_CONTINUE;
@@ -410,13 +385,20 @@ static void *on_udp_interval(oop_source *src, struct timeval tv, void *user)
 		  session->tv_start.tv_sec,
 		  session->test_duration,
 		  session->interval_reads,
-		  session->interval_writes);
+		  session->interval_writes,
+		  slot->interval_pkt_lost,
+		  slot->interval_pkt_dup);
 
   session->byte_interval_recv = 0;
   session->byte_interval_sent = 0;
   session->interval_reads     = 0;
   session->interval_writes    = 0;
 
+  slot->total_pkt_lost += slot->interval_pkt_lost;
+  slot->total_pkt_dup += slot->interval_pkt_dup;
+  slot->interval_pkt_lost = 0;
+  slot->interval_pkt_dup = 0;
+
   schedule_stat_interval(slot);
 
   return OOP_CONTINUE;
@@ -444,6 +426,9 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &slot->session;
   int sd = slot->udp_sd;
 
+  slot->total_pkt_lost += slot->interval_pkt_lost;
+  slot->total_pkt_dup += slot->interval_pkt_dup;
+
   nepim_slot_stat(stdout, NEPIM_LABEL_TOTAL, 
 		  sd, 
 		  slot->index,
@@ -454,7 +439,9 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
 		  session->tv_start.tv_sec,
 		  session->test_duration,
 		  session->total_reads,
-		  session->total_writes);
+		  session->total_writes,
+		  slot->total_pkt_lost,
+		  slot->total_pkt_dup);
 
   session->byte_total_recv = 0;
   session->byte_total_sent = 0;
@@ -1016,6 +1003,8 @@ static int spawn_udp_connection(const char *hostname,
   int result;
   int sd = -1;
 
+  memset(&hints, 0, sizeof(hints));
+
   hints.ai_socktype = SOCK_DGRAM;
   hints.ai_protocol = IPPROTO_UDP;
   hints.ai_flags = AI_CANONNAME;
@@ -1051,7 +1040,9 @@ static int spawn_udp_connection(const char *hostname,
 				     ai->ai_family, ai->ai_socktype, 
 				     ai->ai_protocol,
 				     nepim_global.pmtu_mode,
-				     nepim_global.ttl);
+				     nepim_global.ttl,
+				     nepim_global.win_recv,
+				     nepim_global.win_send);
     if (sd < 0) {
       fprintf(stderr, 
 	      "could not connect UDP socket to %s,%s: %d\n",
@@ -1078,16 +1069,20 @@ static int spawn_udp_connection(const char *hostname,
       } local_sockaddr;
       socklen_t local_sockaddr_len = sizeof(local_sockaddr);
       int result;
+      char local[100];
       
       result = getsockname(sd, (struct sockaddr *) &local_sockaddr,
 			   &local_sockaddr_len);
       assert(!result);
 
+      nepim_sock_dump_addr(local, sizeof(local),
+			   (const struct sockaddr *) &local_sockaddr);
+
       fprintf(stderr, 
 	      "%d: UDP socket (%s,%d) connected to: "
 	      "host=%s,%s(%d) len=%d family=%d type=%d proto=%d\n",
 	      sd, 
-	      inet_ntoa(local_sockaddr.inet.sin_addr),
+	      local,
 	      nepim_sock_get_port((struct sockaddr *) &local_sockaddr),
 	      hostname, portname, 
 	      nepim_sock_get_port(ai->ai_addr),
