@@ -19,7 +19,7 @@ Boston, MA 02111-1307, USA.
 *-GNU-GPL-END-*/
 
 
-/* $Id: udp_server.c,v 1.56 2005/08/17 02:24:29 evertonm Exp $ */
+/* $Id: udp_server.c,v 1.60 2005/09/02 21:57:58 evertonm Exp $ */
 
 
 #include <stdio.h>
@@ -74,13 +74,20 @@ static void *on_udp_interval(oop_source *src, struct timeval tv, void *user)
 		  session->tv_start.tv_sec,
 		  session->test_duration,
 		  session->interval_reads,
-		  session->interval_writes);
+		  session->interval_writes,
+		  slot->interval_pkt_lost,
+		  slot->interval_pkt_dup);
 
   session->byte_interval_recv = 0;
   session->byte_interval_sent = 0;
   session->interval_reads     = 0;
   session->interval_writes    = 0;
 
+  slot->total_pkt_lost += slot->interval_pkt_lost;
+  slot->total_pkt_dup += slot->interval_pkt_dup;
+  slot->interval_pkt_lost = 0;
+  slot->interval_pkt_dup = 0;
+
   schedule_stat_interval(slot->index);
 
   return OOP_CONTINUE;
@@ -112,6 +119,9 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
   nepim_session_t *session = &slot->session;
   int sd = slot->udp_sd;
 
+  slot->total_pkt_lost += slot->interval_pkt_lost;
+  slot->total_pkt_dup += slot->interval_pkt_dup;
+
   nepim_slot_stat(stdout, NEPIM_LABEL_TOTAL, 
 		  sd, 
 		  slot->index,
@@ -122,7 +132,9 @@ static void *on_udp_duration(oop_source *src, struct timeval tv, void *user)
 		  session->tv_start.tv_sec,
 		  session->test_duration,
 		  session->total_reads,
-		  session->total_writes);
+		  session->total_writes,
+		  slot->total_pkt_lost,
+		  slot->total_pkt_dup);
 
   session->byte_total_recv = 0;
   session->byte_total_sent = 0;
@@ -646,6 +658,7 @@ static int parse_greetings(nepim_greet_t *opt,
   int result;
   char *eos;
   char *past_end;
+  char peer[100];
 
   eos = memchr(buf, '\n', buf_len);
   if (eos)
@@ -658,19 +671,19 @@ static int parse_greetings(nepim_greet_t *opt,
 
   *past_end = '\0';
 
-  fprintf(stderr, "DEBUG %s %s udp_buf_len=%d eos_offset=%d\n",
-	  __FILE__, __PRETTY_FUNCTION__, buf_len, past_end - buf);
-
   result = nepim_parse_greetings(opt, buf, past_end);
   if (result)
     return result;
 
+  nepim_sock_dump_addr(peer, sizeof(peer),
+		       (const struct sockaddr *) remote);
+
   fprintf(stderr, 
 	  "%d %d-%d: UDP from (%s,%d): send=%d bps=%lld pps=%d "
 	  "interval=%d duration=%d delay=%ld ka_send=%d ka_req=%d\n", 
 	  sd, 
 	  local_slot, remote_slot,
-	  inet_ntoa(((struct sockaddr_in *) remote)->sin_addr),
+	  peer,
 	  nepim_sock_get_port((struct sockaddr *) remote),
 	  opt->must_send, opt->bit_rate, opt->pkt_rate,
 	  opt->stat_interval, opt->test_duration, 
@@ -680,7 +693,7 @@ static int parse_greetings(nepim_greet_t *opt,
   return 0;
 }
 
-static void udp_consume(nepim_slot_t *slot, size_t len)
+static void udp_consume(nepim_slot_t *slot, size_t len, uint32_t seq)
 {
   nepim_session_t *session = &slot->session;
 
@@ -688,6 +701,9 @@ static void udp_consume(nepim_slot_t *slot, size_t len)
   ++slot->keepalives_recv;
 
   nepim_session_read_add(session, len);
+
+  nepim_win_add(&slot->win_recv_seq, seq,
+		&slot->interval_pkt_lost, &slot->interval_pkt_dup);
 }
 
 static void *on_udp_segment(oop_source *src, int sd,
@@ -750,12 +766,14 @@ static void *on_udp_segment(oop_source *src, int sd,
     int local_slot;
     nepim_greet_t opt;
 
+#if 0
     fprintf(stderr, 
 	    "DEBUG %s %s: new slot request: %d (%d-%d)\n",
 	    __FILE__, __PRETTY_FUNCTION__, 
 	    sd,
 	    nepim_uint16_read(buf + 1),
 	    nepim_uint16_read(buf + 3));
+#endif
 
     if (nepim_slot_find_addr(&slots, hdr.src_slot, 
 			     (const struct sockaddr *) &from,
@@ -827,7 +845,7 @@ static void *on_udp_segment(oop_source *src, int sd,
       return OOP_CONTINUE;
     }
 
-    udp_consume(slot, rd);
+    udp_consume(slot, rd, hdr.seq);
   }
 
   return OOP_CONTINUE;
@@ -877,7 +895,9 @@ int nepim_udp_listener(const char *hostname, const char *portname,
 			     ai->ai_protocol,
 			     nepim_global.pmtu_mode,
 			     nepim_global.ttl,
-			     mcast_join);
+			     mcast_join,
+			     nepim_global.win_recv,
+			     nepim_global.win_send);
     if (sd < 0) {
       fprintf(stderr, 
 	      "spawn_udp_listener: UDP listener socket failed for %s,%s(%d): %d\n",
